use crate::{errors::RunError, stack::StackVec};

pub fn dump(memory: &mut StackVec) {
    if memory.is_empty() {
        println!("Stack is empty.");
        return;
    }

    for (i, entry) in memory.iter().enumerate() {
        println!("{0}: {1}", i, entry);
    }
}

pub fn size(memory: &mut StackVec) {
    memory.push(memory.len() as u16);
}

pub fn maxsize(memory: &mut StackVec) {
    memory.push(memory.size() as u16);
}

pub fn expr(memory: &mut StackVec, arg: String) {
    for code in arg.chars() {
        match code {
            '>' => memory.push(0),
            '$' => { memory.pop(); },
            ':' => {
                let data = memory.last().clone();
                memory.push(data);
            },
            '\\' => {
                let ax = memory.pop();
                let ax2 = memory.pop();

                memory.push(ax);
                memory.push(ax2);
            }
            '+' => {
                let ax = memory.pop();
                memory.push(ax + 1);
            }
            '-' => {
                let ax = memory.pop();
                memory.push(ax - 1);
            }

            _ => {
                eprintln!("{}", RunError::InvalidExpressionUnknownOperator(code.to_string()));
                std::process::exit(2);
            }
        }
    }
}

pub fn exit(data: u16) {
    std::process::exit(data as i32);
}